@@ -23,6 +23,8 @@ mod test_readme {
     external_doc_test!(include_str!("../README.md"));
 }
 
+pub use tagger::upgrade_write;
+
 pub use crop::Crop;
 pub use crop::Croppable;
 mod crop;
@@ -31,9 +33,6 @@ mod render;
 mod util;
 use std::fmt;
 
-///The number of unique colors.
-const NUM_COLORS: usize = 8;
-
 ///The width of the svg tag.
 const WIDTH: f64 = 800.0;
 ///The height of the svg tag.
@@ -115,85 +114,85 @@ pub fn default_svg<T: std::fmt::Write, K>(
 }
 
 /// Default theme using css variables (with light theme defaults if the variables are not set).
-pub const HTML_CONFIG_CSS_VARIABLE_DEFAULT: &str = "<style>.poloto {\
-    stroke-linecap:round;\
-    font-family: sans-serif;\
-    stroke-width:2;\
-    }\
-    .scatter{stroke-width:7}\
-    .poloto_text{fill: var(--poloto_fg_color,black);}\
-    .poloto_axis_lines{stroke: var(--poloto_fg_color,black);stoke-width:3;fill:none}\
-    .poloto_background{fill: var(--poloto_bg_color,aliceblue);}\
-    .poloto0stroke{stroke:  var(--poloto_color0,blue);}\
-    .poloto1stroke{stroke:  var(--poloto_color1,red);}\
-    .poloto2stroke{stroke:  var(--poloto_color2,green);}\
-    .poloto3stroke{stroke:  var(--poloto_color3,gold);}\
-    .poloto4stroke{stroke:  var(--poloto_color4,aqua);}\
-    .poloto5stroke{stroke:  var(--poloto_color5,brown);}\
-    .poloto6stroke{stroke:  var(--poloto_color6,lime);}\
-    .poloto7stroke{stroke:  var(--poloto_color7,chocolate);}\
-    .poloto0fill{fill:var(--poloto_color0,blue);}\
-    .poloto1fill{fill:var(--poloto_color1,red);}\
-    .poloto2fill{fill:var(--poloto_color2,green);}\
-    .poloto3fill{fill:var(--poloto_color3,gold);}\
-    .poloto4fill{fill:var(--poloto_color4,aqua);}\
-    .poloto5fill{fill:var(--poloto_color5,brown);}\
-    .poloto6fill{fill:var(--poloto_color6,lime);}\
-    .poloto7fill{fill:var(--poloto_color7,chocolate);}</style>";
+pub const HTML_CONFIG_CSS_VARIABLE_DEFAULT: &str = r###"<style>.poloto {
+    stroke-linecap:round;
+    font-family: sans-serif;
+    stroke-width:2;
+    }
+    .scatter{stroke-width:7}
+    .poloto_text{fill: var(--poloto_fg_color,black);}
+    .poloto_axis_lines{stroke: var(--poloto_fg_color,black);stroke-width:3;fill:none}
+    .poloto_background{fill: var(--poloto_bg_color,aliceblue);}
+    .poloto0stroke{stroke:  var(--poloto_color0,blue);}
+    .poloto1stroke{stroke:  var(--poloto_color1,red);}
+    .poloto2stroke{stroke:  var(--poloto_color2,green);}
+    .poloto3stroke{stroke:  var(--poloto_color3,gold);}
+    .poloto4stroke{stroke:  var(--poloto_color4,aqua);}
+    .poloto5stroke{stroke:  var(--poloto_color5,brown);}
+    .poloto6stroke{stroke:  var(--poloto_color6,lime);}
+    .poloto7stroke{stroke:  var(--poloto_color7,chocolate);}
+    .poloto0fill{fill:var(--poloto_color0,blue);}
+    .poloto1fill{fill:var(--poloto_color1,red);}
+    .poloto2fill{fill:var(--poloto_color2,green);}
+    .poloto3fill{fill:var(--poloto_color3,gold);}
+    .poloto4fill{fill:var(--poloto_color4,aqua);}
+    .poloto5fill{fill:var(--poloto_color5,brown);}
+    .poloto6fill{fill:var(--poloto_color6,lime);}
+    .poloto7fill{fill:var(--poloto_color7,chocolate);}</style>"###;
 
 /// Default light theme
-pub const HTML_CONFIG_LIGHT_DEFAULT: &str = "<style>.poloto {\
-    stroke-linecap:round;\
-    font-family: sans-serif;\
-    stroke-width:2;\
-    }\
-    .scatter{stroke-width:7}\
-    .poloto_text{fill: black;}\
-    .poloto_axis_lines{stroke: black;stoke-width:3;fill:none}\
-    .poloto_background{fill: aliceblue;}\
-    .poloto0stroke{stroke:  blue;}\
-    .poloto1stroke{stroke:  red;}\
-    .poloto2stroke{stroke:  green;}\
-    .poloto3stroke{stroke:  gold;}\
-    .poloto4stroke{stroke:  aqua;}\
-    .poloto5stroke{stroke:  brown;}\
-    .poloto6stroke{stroke:  lime;}\
-    .poloto7stroke{stroke:  chocolate;}\
-    .poloto0fill{fill:blue;}\
-    .poloto1fill{fill:red;}\
-    .poloto2fill{fill:green;}\
-    .poloto3fill{fill:gold;}\
-    .poloto4fill{fill:aqua;}\
-    .poloto5fill{fill:brown;}\
-    .poloto6fill{fill:lime;}\
-    .poloto7fill{fill:chocolate;}</style>";
+pub const HTML_CONFIG_LIGHT_DEFAULT: &str = r###"<style>.poloto {
+    stroke-linecap:round;
+    font-family: sans-serif;
+    stroke-width:2;
+    }
+    .scatter{stroke-width:7}
+    .poloto_text{fill: black;}
+    .poloto_axis_lines{stroke: black;stroke-width:3;fill:none}
+    .poloto_background{fill: aliceblue;}
+    .poloto0stroke{stroke:  blue;}
+    .poloto1stroke{stroke:  red;}
+    .poloto2stroke{stroke:  green;}
+    .poloto3stroke{stroke:  gold;}
+    .poloto4stroke{stroke:  aqua;}
+    .poloto5stroke{stroke:  brown;}
+    .poloto6stroke{stroke:  lime;}
+    .poloto7stroke{stroke:  chocolate;}
+    .poloto0fill{fill:blue;}
+    .poloto1fill{fill:red;}
+    .poloto2fill{fill:green;}
+    .poloto3fill{fill:gold;}
+    .poloto4fill{fill:aqua;}
+    .poloto5fill{fill:brown;}
+    .poloto6fill{fill:lime;}
+    .poloto7fill{fill:chocolate;}</style>"###;
 
 /// Default dark theme
-pub const HTML_CONFIG_DARK_DEFAULT: &str = "<style>.poloto {\
-    stroke-linecap:round;\
-    font-family: sans-serif;\
-    stroke-width:2;\
-    }\
-    .scatter{stroke-width:7}\
-    .poloto_text{fill: white;}\
-    .poloto_axis_lines{stroke: white;stoke-width:3;fill:none}\
-    .poloto_background{fill: black;}\
-    .poloto0stroke{stroke:  blue;}\
-    .poloto1stroke{stroke:  red;}\
-    .poloto2stroke{stroke:  green;}\
-    .poloto3stroke{stroke:  gold;}\
-    .poloto4stroke{stroke:  aqua;}\
-    .poloto5stroke{stroke:  brown;}\
-    .poloto6stroke{stroke:  lime;}\
-    .poloto7stroke{stroke:  chocolate;}\
-    .poloto0fill{fill:blue;}\
-    .poloto1fill{fill:red;}\
-    .poloto2fill{fill:green;}\
-    .poloto3fill{fill:gold;}\
-    .poloto4fill{fill:aqua;}\
-    .poloto5fill{fill:brown;}\
-    .poloto6fill{fill:lime;}\
-    .poloto7fill{fill:chocolate;}</style>";
+pub const HTML_CONFIG_DARK_DEFAULT: &str = r###"<style>.poloto {
+    stroke-linecap:round;
+    font-family: sans-serif;
+    stroke-width:2;
+    }
+    .scatter{stroke-width:7}
+    .poloto_text{fill: white;}
+    .poloto_axis_lines{stroke: white;stroke-width:3;fill:none}
+    .poloto_background{fill: black;}
+    .poloto0stroke{stroke:  blue;}
+    .poloto1stroke{stroke:  red;}
+    .poloto2stroke{stroke:  green;}
+    .poloto3stroke{stroke:  gold;}
+    .poloto4stroke{stroke:  aqua;}
+    .poloto5stroke{stroke:  brown;}
+    .poloto6stroke{stroke:  lime;}
+    .poloto7stroke{stroke:  chocolate;}
+    .poloto0fill{fill:blue;}
+    .poloto1fill{fill:red;}
+    .poloto2fill{fill:green;}
+    .poloto3fill{fill:gold;}
+    .poloto4fill{fill:aqua;}
+    .poloto5fill{fill:brown;}
+    .poloto6fill{fill:lime;}
+    .poloto7fill{fill:chocolate;}</style>"###;
 
 /// The demsions of the svg graph `[800,500]`.
 pub const DIMENSIONS: [usize; 2] = [800, 500];
@@ -296,6 +295,7 @@ pub struct Plotter<'a> {
     plots: Vec<Plot<'a>>,
     xmarkers: Vec<f64>,
     ymarkers: Vec<f64>,
+    num_css_classes: Option<usize>,
 }
 
 impl<'a> Plotter<'a> {
@@ -317,6 +317,7 @@ impl<'a> Plotter<'a> {
             plots: Vec::new(),
             xmarkers: Vec::new(),
             ymarkers: Vec::new(),
+            num_css_classes: Some(8),
         }
     }
     /// Create a line from plots using a SVG polyline element.
@@ -448,8 +449,27 @@ impl<'a> Plotter<'a> {
         self
     }
 
+    ///
+    /// The number of distinct css classes. If there are more plots than
+    /// classes, then they will wrap around. The default value is 8.
+    ///
+    /// A value of None, means it will never wrap around.
+    ///
+    /// ```
+    /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
+    /// let mut plotter = poloto::plot("title", "x", "y");
+    /// plotter.line("", &data);
+    /// plotter.num_css_class(Some(30));
+    /// ```
+    ///
+    pub fn num_css_class(&mut self, a: Option<usize>) -> &mut Self {
+        self.num_css_classes = a;
+        self
+    }
+
     ///
     /// Use the plot iterators to write out the graph elements.
+    /// Does not add a svg tag, or any styling elements.
     ///
     /// Panics if the render fails.
     ///
@@ -461,36 +481,48 @@ impl<'a> Plotter<'a> {
     /// let mut plotter = poloto::plot("title", "x", "y");
     /// plotter.line("", &data);
     /// let mut k=String::new();
-    /// plotter.render(tagger::new(&mut k));
+    /// plotter.render(&mut k);
     /// ```
-    pub fn render<T: std::fmt::Write, K>(&mut self, mut a: K) -> K
-    where
-        K: std::borrow::BorrowMut<tagger::ElemWriter<T>>,
-    {
-        render::render(self, a.borrow_mut());
-        a
+    pub fn render<T: std::fmt::Write>(&mut self, a: T) -> T {
+        render::render(self, a)
     }
 
-    pub fn simple_theme<T: std::fmt::Write, K>(&mut self, mut a: K) -> K
-    where
-        K: std::borrow::BorrowMut<tagger::ElemWriter<T>>,
-    {
-        default_svg(a.borrow_mut(), tagger::no_attr(), |d| {
-            d.put_raw(HTML_CONFIG_LIGHT_DEFAULT);
-            self.render(d);
+    ///
+    /// Make a graph with a svg tag and a simple dark css theme.
+    ///
+    /// ```
+    /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
+    /// let mut plotter = poloto::plot("title", "x", "y");
+    /// plotter.line("", &data);
+    /// let mut k=String::new();
+    /// plotter.simple_theme(&mut k);
+    /// ```
+    pub fn simple_theme<T: std::fmt::Write>(&mut self, a: T) -> T {
+        let mut w = tagger::new(a);
+        default_svg(&mut w, tagger::no_attr(), |d| {
+            d.put_raw(minify(HTML_CONFIG_LIGHT_DEFAULT));
+            self.render(d.writer());
         });
-        a
+        w.into_writer()
     }
 
-    pub fn simple_theme_dark<T: std::fmt::Write, K>(&mut self, mut a: K) -> K
-    where
-        K: std::borrow::BorrowMut<tagger::ElemWriter<T>>,
-    {
-        default_svg(a.borrow_mut(), tagger::no_attr(), |d| {
-            d.put_raw(HTML_CONFIG_DARK_DEFAULT);
-            self.render(d);
+    ///
+    /// Make a graph with a svg tag and a simple dark css theme.
+    ///
+    /// ```
+    /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
+    /// let mut plotter = poloto::plot("title", "x", "y");
+    /// plotter.line("", &data);
+    /// let mut k=String::new();
+    /// plotter.simple_theme_dark(&mut k);
+    /// ```
+    pub fn simple_theme_dark<T: std::fmt::Write>(&mut self, a: T) -> T {
+        let mut w = tagger::new(a);
+        default_svg(&mut w, tagger::no_attr(), |d| {
+            d.put_raw(minify(HTML_CONFIG_DARK_DEFAULT));
+            self.render(d.writer());
         });
-        a
+        w.into_writer()
     }
 }
 
@@ -499,19 +531,34 @@ impl<'a> Plotter<'a> {
 #[macro_export]
 macro_rules! formatm {
     ($($arg:tt)*) => {
-        $crate::moveable_format(move |w| write!(w,$($arg)*))
+        $crate::DisplayableClosure::new(move |w| write!(w,$($arg)*))
     }
 }
 
 /// Convert a moved closure into a impl fmt::Display.
 /// This is useful because std's `format_args!()` macro
 /// has a shorter lifetime.
-pub fn moveable_format(func: impl Fn(&mut fmt::Formatter) -> fmt::Result) -> impl fmt::Display {
-    struct Foo<F>(F);
-    impl<F: Fn(&mut fmt::Formatter) -> fmt::Result> fmt::Display for Foo<F> {
-        fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
-            (self.0)(formatter)
-        }
+pub struct DisplayableClosure<F>(pub F);
+
+impl<F: Fn(&mut fmt::Formatter) -> fmt::Result> DisplayableClosure<F> {
+    pub fn new(a: F) -> Self {
+        DisplayableClosure(a)
     }
-    Foo(func)
+}
+impl<F: Fn(&mut fmt::Formatter) -> fmt::Result> fmt::Display for DisplayableClosure<F> {
+    fn fmt(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+        (self.0)(formatter)
+    }
+}
+
+///
+/// Remove all whitespace/newlines from a tring.
+///
+pub fn minify(a: &str) -> impl fmt::Display + '_ {
+    DisplayableClosure::new(move |w| {
+        for a in a.split_whitespace() {
+            write!(w, "{}", a)?;
+        }
+        Ok(())
+    })
 }
